@@ -142,39 +142,54 @@ sysprof_elf_symbol_resolver_load (SysprofSymbolResolver *resolver,
 
   while (sysprof_capture_reader_peek_type (reader, &type))
     {
-      const SysprofCaptureMap *ev;
-      SysprofMapLookaside *lookaside;
-      SysprofMap map;
+      if (type == SYSPROF_CAPTURE_FRAME_MAP)
+        {
+          const SysprofCaptureMap *ev = sysprof_capture_reader_read_map (reader);
+          SysprofMapLookaside *lookaside = g_hash_table_lookup (self->lookasides, GINT_TO_POINTER (ev->frame.pid));
+          SysprofMap map;
 
-      if (type != SYSPROF_CAPTURE_FRAME_MAP)
+          map.start = ev->start;
+          map.end = ev->end;
+          map.offset = ev->offset;
+          map.inode = ev->inode;
+          map.filename = ev->filename;
+
+          if (lookaside == NULL)
+            {
+              lookaside = sysprof_map_lookaside_new ();
+              g_hash_table_insert (self->lookasides, GINT_TO_POINTER (ev->frame.pid), lookaside);
+            }
+
+          sysprof_map_lookaside_insert (lookaside, &map);
+        }
+      else if (type == SYSPROF_CAPTURE_FRAME_OVERLAY)
+        {
+          const SysprofCaptureOverlay *ev = sysprof_capture_reader_read_overlay (reader);
+          SysprofMapLookaside *lookaside = g_hash_table_lookup (self->lookasides, GINT_TO_POINTER (ev->frame.pid));
+          const char *src = ev->data;
+          const char *dst = &ev->data[ev->src_len+1];
+
+          if (lookaside == NULL)
+            {
+              lookaside = sysprof_map_lookaside_new ();
+              g_hash_table_insert (self->lookasides, GINT_TO_POINTER (ev->frame.pid), lookaside);
+            }
+
+          sysprof_map_lookaside_overlay (lookaside, src, dst);
+        }
+      else
         {
           if (!sysprof_capture_reader_skip (reader))
             return;
           continue;
         }
-
-      ev = sysprof_capture_reader_read_map (reader);
-
-      map.start = ev->start;
-      map.end = ev->end;
-      map.offset = ev->offset;
-      map.inode = ev->inode;
-      map.filename = ev->filename;
-
-      lookaside = g_hash_table_lookup (self->lookasides, GINT_TO_POINTER (ev->frame.pid));
-
-      if (lookaside == NULL)
-        {
-          lookaside = sysprof_map_lookaside_new ();
-          g_hash_table_insert (self->lookasides, GINT_TO_POINTER (ev->frame.pid), lookaside);
-        }
-
-      sysprof_map_lookaside_insert (lookaside, &map);
     }
 }
 
 static bin_file_t *
 sysprof_elf_symbol_resolver_get_bin_file (SysprofElfSymbolResolver *self,
+                                          const SysprofMapOverlay  *overlays,
+                                          guint                     n_overlays,
                                           const gchar              *filename)
 {
   bin_file_t *bin_file;
@@ -185,31 +200,29 @@ sysprof_elf_symbol_resolver_get_bin_file (SysprofElfSymbolResolver *self,
 
   if (bin_file == NULL)
     {
+      g_autofree gchar *path = NULL;
       const gchar *alternate = filename;
       const gchar * const *dirs;
 
       dirs = (const gchar * const *)(gpointer)self->debug_dirs->data;
 
-      /*
-       * If we are in a new mount namespace, then rely on the sysprof_symbol_dirs
-       * to find us a locate to resolve the file where the CRC will match.
-       *
-       * TODO: We need to translate the path here so that we can locate the
-       *       binary behind it (which then has links to the debug file in
-       *       the section header).
-       */
-      if (g_str_has_prefix (filename, "/newroot/"))
-        alternate += strlen ("/newroot");
+      if (overlays && filename[0] != '/' && filename[0] != '[')
+        {
+          for (guint i = 0; i < n_overlays; i++)
+            {
+              if (g_str_has_prefix (filename, overlays[i].dst+1))
+                {
+                  alternate = path = g_build_filename (overlays[i].src, filename, NULL);
+                  break;
+                }
+            }
+        }
+      else if (is_flatpak () && g_str_has_prefix (filename, "/usr/"))
+        {
+          alternate = path = g_build_filename ("/var/run/host", alternate, NULL);
+        }
 
-      if (is_flatpak () && g_str_has_prefix (filename, "/usr/"))
-        {
-          g_autofree gchar *path = g_build_filename ("/var/run/host", alternate, NULL);
-          bin_file = bin_file_new (path, dirs);
-        }
-      else
-        {
-          bin_file = bin_file_new (alternate, dirs);
-        }
+      bin_file = bin_file_new (alternate, dirs);
 
       g_hash_table_insert (self->bin_files, g_strdup (filename), bin_file);
     }
@@ -336,12 +349,14 @@ sysprof_elf_symbol_resolver_resolve_full (SysprofElfSymbolResolver *self,
                                           GQuark                   *tag)
 {
   SysprofMapLookaside *lookaside;
+  const SysprofMapOverlay *overlays = NULL;
   const bin_symbol_t *bin_sym;
   const gchar *bin_sym_name;
   const SysprofMap *map;
   bin_file_t *bin_file;
   gulong ubegin;
   gulong uend;
+  guint n_overlays = 0;
 
   g_assert (SYSPROF_IS_ELF_SYMBOL_RESOLVER (self));
   g_assert (name != NULL);
@@ -364,7 +379,13 @@ sysprof_elf_symbol_resolver_resolve_full (SysprofElfSymbolResolver *self,
   address -= map->start;
   address += map->offset;
 
-  bin_file = sysprof_elf_symbol_resolver_get_bin_file (self, map->filename);
+  if (lookaside->overlays)
+    {
+      overlays = &g_array_index (lookaside->overlays, SysprofMapOverlay, 0);
+      n_overlays = lookaside->overlays->len;
+    }
+
+  bin_file = sysprof_elf_symbol_resolver_get_bin_file (self, overlays, n_overlays, map->filename);
 
   g_assert (bin_file != NULL);
 
